@@ -7,9 +7,9 @@ use warnings;
 package Org::Parser::Tiny;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-02-05'; # DATE
+our $DATE = '2020-02-06'; # DATE
 our $DIST = 'Org-Parser-Tiny'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 sub new {
     my $class = shift;
@@ -60,7 +60,7 @@ sub _parse {
             my $level = length($1);
             my $title = $2;
             my $node = Org::Parser::Tiny::Node::Headline->new(
-                raw   => $line,
+                _str  => $line,
                 level => $level,
             );
 
@@ -164,8 +164,8 @@ sub new {
 
 sub parent { $_[0]{parent} }
 sub children { $_[0]{children} || [] }
-sub as_string { $_[0]{raw} }
-
+sub as_string { $_[0]{_str} }
+sub children_as_string { join("", map { $_->as_string } @{ $_[0]->children }) }
 
 # abstract class: Org::Parser::Tiny::HasPreamble
 package Org::Parser::Tiny::Node::HasPreamble;
@@ -178,33 +178,64 @@ sub new {
     $class->SUPER::new(%args);
 }
 
-sub as_string {
-    $_[0]->{preamble} . join("", map { $_->as_string } @{ $_[0]->children });
-}
-
 
 # class: Org::Parser::Tiny::Document: top level node
 package Org::Parser::Tiny::Node::Document;
 
 our @ISA = qw(Org::Parser::Tiny::Node::HasPreamble);
 
+sub as_string {
+    $_[0]->{preamble} . $_[0]->children_as_string;
+}
+
 
 # class: Org::Parser::Tiny::Node::Headline: headline with its content
 package Org::Parser::Tiny::Node::Headline;
 
 our @ISA = qw(Org::Parser::Tiny::Node::HasPreamble);
 
-sub level { $_[0]{level} }
+sub level {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{level} = $_[1] }
+    $_[0]{level};
+}
 
-sub title { $_[0]{title} }
+sub title {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{title} = $_[1] }
+    $_[0]{title};
+}
 
-sub is_todo { $_[0]{is_todo} }
+sub is_todo {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{is_todo} = $_[1] }
+    $_[0]{is_todo};
+}
 
-sub is_done { $_[0]{is_done} }
+sub is_done {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{is_done} = $_[1] }
+    $_[0]{is_done};
+}
 
-sub todo_state { $_[0]{todo_state} }
+sub todo_state {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{todo_state} = $_[1] }
+    $_[0]{todo_state};
+}
 
-sub tags { $_[0]{tags} || [] }
+sub tags {
+    if (@_ > 1) { undef $_[0]{_str}; $_[0]{tags} = $_[1] }
+    $_[0]{tags} || [];
+}
+
+sub as_string {
+    ($_[0]->{_str} //
+         join('',
+              "*" x $_[0]{level},
+              " ",
+              (length $_[0]{todo_state} ? "$_[0]{todo_state} " : ""),
+              "$_[0]{title}",
+              (defined $_[0]{tags} ? " :".join(":", @{ $_[0]{tags} }).":" : ""),
+              "\n",
+          )) .
+              $_[0]->{preamble};
+}
 
 1;
 # ABSTRACT: Parse Org documents with as little code (and no non-core deps) as possible
@@ -221,7 +252,7 @@ Org::Parser::Tiny - Parse Org documents with as little code (and no non-core dep
 
 =head1 VERSION
 
-This document describes version 0.002 of Org::Parser::Tiny (from Perl distribution Org-Parser-Tiny), released on 2020-02-05.
+This document describes version 0.003 of Org::Parser::Tiny (from Perl distribution Org-Parser-Tiny), released on 2020-02-06.
 
 =head1 SYNOPSIS
 
